package com.mortals.xhx.module.user.service.impl;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.utils.BeanUtil;
import com.mortals.xhx.common.utils.StringUtils;
import com.mortals.xhx.module.datum.model.DatumInfoFieldEntity;
import com.mortals.xhx.module.datum.model.DatumInfoFieldQuery;
import com.mortals.xhx.module.datum.service.DatumInfoFieldService;
import com.mortals.xhx.module.matter.model.MatterDatumEntity;
import com.mortals.xhx.module.matter.service.MatterDatumService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.user.dao.UserFillHistoryDao;
import com.mortals.xhx.module.user.model.UserFillHistoryEntity;
import com.mortals.xhx.module.user.service.UserFillHistoryService;
import lombok.extern.slf4j.Slf4j;
/**
* UserFillHistoryService
* 用户填单记录 service实现
*
* @author zxfei
* @date 2025-06-08
*/
@Service("userFillHistoryService")
@Slf4j
public class UserFillHistoryServiceImpl extends AbstractCRUDServiceImpl<UserFillHistoryDao, UserFillHistoryEntity, Long> implements UserFillHistoryService {

    @Autowired
    private MatterDatumService matterDatumService;
    @Autowired
    private DatumInfoFieldService datumInfoFieldService;

    @Override
    protected void validData(UserFillHistoryEntity entity, Context context) throws AppException {
        if(entity.getMaterialId()==null){
            throw new AppException("材料id不能为空");
        }
        MatterDatumEntity datumEntity = matterDatumService.get(entity.getMaterialId());
        if(datumEntity==null){
            throw new AppException("材料id错误");
        }
        entity.setMaterialName(datumEntity.getMaterialName());
        entity.setMatterId(datumEntity.getMatterId());
        entity.setMatterName(datumEntity.getMatterName());
        entity.setPreViewPath(datumEntity.getPreViewPath());
        if(StringUtils.isNotEmpty(entity.getFormValue())){
            JSONObject value = JSONObject.parseObject(entity.getFormValue());
            JSONObject formTemplate = JSONObject.parseObject(datumEntity.getFormContent());
            JSONArray formList = formTemplate.getJSONArray("list");
            for (int j = 0; j < formList.size(); j++) {
                JSONObject jsonObject = formList.getJSONObject(j);
                String key = jsonObject.getString("id");
                if(value.containsKey(key)){
                    jsonObject.put("value",value.getString(key));
                }
            }
            entity.setFormContent(formTemplate.toJSONString());
        }
    }

    @Override
    protected void saveAfter(UserFillHistoryEntity entity, Context context) throws AppException {
        saveUserDatumField(entity);
    }
    @Override
    protected void updateAfter(UserFillHistoryEntity entity, Context context) throws AppException {
        saveUserDatumField(entity);
    }

    private void saveUserDatumField(UserFillHistoryEntity entity){
        if(StringUtils.isNotEmpty(entity.getFormContent())){
            List<DatumInfoFieldEntity> datumInfoFieldList = datumInfoFieldService.find(new DatumInfoFieldQuery().openId(entity.getOpenId()).matterId(entity.getMatterId()).materialId(entity.getMaterialId()));
            Map<String,DatumInfoFieldEntity> fieldMap = new HashMap<>();
            if(CollectionUtils.isNotEmpty(datumInfoFieldList)){
                fieldMap = datumInfoFieldList.stream().collect(Collectors.toMap(i->i.getFieldCode(),Function.identity()));
            }
            List<DatumInfoFieldEntity> updateList = new ArrayList<>();
            List<DatumInfoFieldEntity> addList = new ArrayList<>();
            JSONObject formTemplate = JSONObject.parseObject(entity.getFormContent());
            JSONArray formList = formTemplate.getJSONArray("list");
            for (int j = 0; j < formList.size(); j++) {
                JSONObject jsonObject = formList.getJSONObject(j);
                String key = jsonObject.getString("id");
                String value = jsonObject.getString("value");
                if(fieldMap.containsKey(key)){
                    fieldMap.get(key).setFieldValue(value);
                    fieldMap.get(key).setUpdateUserId(1L);
                    fieldMap.get(key).setUpdateTime(new Date());
                    updateList.add(fieldMap.get(key));
                }else {
                    DatumInfoFieldEntity newEntity = new DatumInfoFieldEntity();
                    newEntity.initAttrValue();
                    BeanUtils.copyProperties(entity,newEntity, BeanUtil.getNullPropertyNames(entity));
                    newEntity.setFieldCode(key);
                    newEntity.setFieldValue(value);
                    newEntity.setFieldName(jsonObject.getString("label"));
                    newEntity.setFieldLen(jsonObject.getIntValue("maxLength"));
                    newEntity.setFieldNull(jsonObject.getIntValue("required"));
                    //newEntity.setFieldOrderNo(jsonObject.getString("label"));
                    newEntity.setFieldType(jsonObject.getString("compType"));
                    newEntity.setFieldTypeValue(jsonObject.getString("label"));
                    addList.add(newEntity);
                }
            }
            if(updateList.size()>0){
                datumInfoFieldService.update(updateList);
            }
            if(addList.size()>0){
                datumInfoFieldService.save(addList);
            }
        }
    }
}